/**
 * @file
 * Provides JavaScript additions to replace insert block.
 *
 * This file provides insert block in ckeditor.
 */

(function ($, drupalSettings, Drupal, once) {

  "use strict";

  /**
   * Attach behaviors to insert block for ckeditor.
   */
  Drupal.behaviors.insertBlock = {
    attach: function (context, settings) {
      // Base URL for fetching block content.
      let baseUrl = drupalSettings.path.baseUrl + 'get-block-content/';

      // Process elements with the class "insert-block".
      once('insert-block', '.insert-block', context).forEach((element) => {
        // Get the block ID from the data attribute.
        let blockId = $(element).data('block-id');
        if (blockId) {
          // Construct the full URL.
          let url = baseUrl + blockId;
          // Fetch block content and insert it into the element.
          fetch(url)
            .then((response) => {
              if (!response.ok) {
                throw new Error('Network response was not ok: ' + response.statusText);
              }
              return response.text();
            })
            .then((content) => {
              $(element).html(content);
            })
            .catch((error) => {
              console.error('Error fetching block content:', error);
            });
        }
      });
    }
  };

})(jQuery,drupalSettings, Drupal, once);
